#include "../include/aprecv-general.h"

int print_llc_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size, unsigned short length) {
  static struct llc_pkt_header *llc_header;
  static unsigned short llc_snap_type=0;
  static unsigned int oui=0;

  llc_header = (struct llc_pkt_header *)(packet+data_link_size+opt_size);
  
  apsr_output("\n- LLC Header -");

  opt_size += 3;
  if(expand_llc || expand_all) {
    apsr_output("\n  DSAP: 0x%02x", llc_header->dsap);
    if(verbose) {
            switch(llc_header->ssap) {
                    case APSR_SAP_SNAP:
	              apsr_output(" (SNAP)");
	              break;
	            case APSR_SAP_NETWARE:
	              apsr_output(" (Novell Netware)");
                      break;
                    default:
                      apsr_output(" (unknown)");
                      break;
	    }
    }		
    if(llc_header->dsap & 0x01) {
	    apsr_output(", IG Bit: 1");
	    if(verbose)
		    apsr_output(" (Group)");
    }
    else {
	    apsr_output(", IG Bit: 0"); 
	    if(verbose)
		    apsr_output(" (Individual)");
    }	    
    apsr_output("\n  SSAP: 0x%02x", llc_header->ssap);
    if(verbose) {
	    switch(llc_header->ssap) {
		    case APSR_SAP_SNAP:
		      apsr_output(" (SNAP)");
		      break;
		    case APSR_SAP_NETWARE:
		      apsr_output(" (Novell Netware)");
		      break;
		    default:
		      apsr_output(" (unknown)");
		      break;
            }		    
    }		  
    if(llc_header->ssap & 0x01) {
	    apsr_output(", CR-Bit: 1");
	    if(verbose) 
		    apsr_output(" (Response)");
    }
    else {
	    apsr_output(", CR-Bit: 0");
	    if(verbose)
		    apsr_output(" (Command)");
    }
    if(llc_header->ssap & 0x02) {
	    apsr_output(", U-Bit: 1");
	    if(verbose) 
		    apsr_output(" (IEEE)");
    }
    else {
	    apsr_output(", U-Bit: 0");
	    if(verbose)
		    apsr_output(" (User)");
    }
    apsr_output("\n  Control: 0x%02x", llc_header->control);
    if(verbose) {
	    if(llc_header->control & 0x01 &&
	       llc_header->control & 0x02)
		    apsr_output(" (Unnumbered frame)");
	    else if(llc_header->control & 0x01 &&
		    !(llc_header->control & 0x02))
		    apsr_output(" (Supervisory frame)");
	    else if(!(llc_header->control & 0x01)) 
		    apsr_output(" (I frame)");

	    if(llc_header->control == APSR_CONTROL_TEST &&
	       llc_header->ssap == APSR_SAP_TEST) 
		    apsr_output(" (LLCPing, Poll)");
	    else if(llc_header->control == APSR_CONTROL_TEST &&
		    llc_header->ssap == APSR_SAP_TEST)
		    apsr_output(" (LLCPing, Final)");
    }
  }
    
  if(llc_header->dsap == APSR_SAP_NETWARE && 
		  llc_header->ssap == APSR_SAP_NETWARE &&
                  llc_header->control == APSR_CONTROL_UI) {
    /* Novel Netware */
    print_ipx_packet(packet, opt_size);
  }
  if(llc_header->dsap == APSR_SAP_TCPIP &&
		  llc_header->ssap == APSR_SAP_TCPIP) {
    /* TCP/IP */
    /* Don't understand this, i need +1 for opt_size because Control 
     * is 2 bytes ??? */
    print_ipv4_packet(header, packet, opt_size+1);
  }
  else if(llc_header->dsap == APSR_SAP_SNAP && 
		  llc_header->ssap == APSR_SAP_SNAP && 
		  llc_header->control == APSR_CONTROL_UI) {                 
    opt_size += 5;
   
    llc_snap_type = apsr_ntohs(llc_header->type);
    oui = APSR_24TO32(llc_header->org_id);
	    
    
    if(expand_llc || expand_all) {
      apsr_output("\n  OUI: 0x%06x", oui);

      if(verbose)
        print_snap_oui(oui);
    }
 
    if(expand_llc || expand_all) {
      apsr_output("\n  Type: 0x%04x", llc_snap_type);
              
      if(verbose) {
        if(oui == APSR_SNAP_OUI_CISCO) {
	       if(llc_snap_type == APSR_SNAP_PID_CDP)
		       apsr_output(" (CDP)");
	       else if(llc_snap_type == APSR_SNAP_PID_CGMP) 
		       apsr_output(" (CGMP)");
	       else if(llc_snap_type == APSR_SNAP_PID_VTP)
	               apsr_output(" (VTP)");
	}
	else if(oui == APSR_SNAP_OUI_ATMFORUM) {
	       if(llc_snap_type == APSR_SNAP_PID_LUNI_CD) 
		       apsr_output(" (LE configuration direct)");
	       else if(llc_snap_type == APSR_SNAP_PID_LUNI_DD3)
		       apsr_output(" (LE data direct 802.3)");
               else if(llc_snap_type == APSR_SNAP_PID_LUNI_DD5)
                       apsr_output(" (LE data direct 802.5)");
               else if(llc_snap_type == APSR_SNAP_PID_LUNI_M3)
                       apsr_output(" (LE multicast 802.3)");
               else if(llc_snap_type == APSR_SNAP_PID_LUNI_M5)
                       apsr_output(" (LE multicast 802.5)");
	}
	else if(oui == APSR_SNAP_OUI_ENCETH || oui == APSR_SNAP_OUI_CISCO_90)
   	  print_ethernet_proto(llc_snap_type);
        else
          apsr_output(" (unknown)");
      }
    }
       
    if(statistic) 
      stats.c.ethernet8023snap++;

    if(oui == APSR_SNAP_OUI_CISCO) {
	   if(llc_snap_type == APSR_SNAP_PID_CDP)
		   print_cdp_packet(header, packet, opt_size, length-opt_size);
	   else if(llc_snap_type == APSR_SNAP_PID_CGMP)
		   print_cgmp_packet(packet, opt_size);
	   else if(llc_snap_type == APSR_SNAP_PID_VTP)
	           print_vtp_packet(packet, opt_size);
    }
    else if(oui == APSR_SNAP_OUI_ATMFORUM) {
	    if(statistic)
		    stats.c.luni++;
    }
    else if(oui == APSR_SNAP_OUI_ENCETH || oui == APSR_SNAP_OUI_CISCO_90) 
	    switchto_ethernet_proto(header, packet, opt_size, llc_snap_type);
    else
      apsr_output("\n");
  }
  else {
    apsr_output("\n");
    if(statistic)
       stats.c.ethernet8023++;
  }

  return(RET_SUCCESS);
}

void print_snap_oui(unsigned int orgcode) {

  switch(orgcode) {
	  case APSR_SNAP_OUI_ENCETH:
            apsr_output(" (Encapsulated ethernet)");
            break;
	  case APSR_SNAP_OUI_CISCO:
	    apsr_output(" (Cisco protocols)");
	    break;
          case APSR_SNAP_OUI_CISCO_90:
	    apsr_output(" (Cisco bridging)");
	    break;
	  case APSR_SNAP_OUI_BRIETH:
	    apsr_output(" (Bridged ethernet)");
	    break;
	  case APSR_SNAP_OUI_APPLETALK:
	    apsr_output(" (Appletalk)");
	    break;
          case APSR_SNAP_OUI_ATMFORUM:
            apsr_output(" (ATM Forum)");
            break;
	  default:
	    apsr_output(" (unkown)");
	    break;
  }
}  
