/* Typedefs for callbacks */
typedef void (*apsr_sniff_callback)(struct sockaddr *, unsigned char *,
				    unsigned int);

typedef void (*apsr_read_bluetooth_callback)(unsigned char *, unsigned int);

typedef void (*apsr_exit_callback)(int);


/* Quiet mode */
volatile unsigned int APSR_QUIET;
volatile unsigned int APSR_RQUIET;


/* Prototypes */

/* apsrlib-dummy.c   */
int apsr_dummy			(void);			

/* apsrlib-ip.c      */
int apsr_init_psize		(int, unsigned char **);
int apsr_free_psize		(unsigned char **);	

/* apsrlib-generic.c */
unsigned long apsr_resolve      (char *);		
unsigned int apsr_conv_mac	(char *);		
char * apsr_resolve_mac	        (unsigned char *);	
int apsr_open_io_socket		(void);			
int apsr_close_io_socket	(int);			
const char *ether2str		(unsigned char *);	
const char *apsr_get_network_address	(struct in_addr, int); 
unsigned short int apsr_htons	(unsigned short int);	
unsigned long int apsr_htonl	(unsigned long int);	
unsigned short int apsr_ntohs	(unsigned short int);	
unsigned long int apsr_ntohl	(unsigned long int);	
const char * apsr_resolve_port	(int, char *);		
int apsr_toggle_promisc		(char *, int);		
int apsr_get_link_bandwidth	(const char *, int);	
void apsr_setnonblocking	(int);			
#ifdef HAVE_ICONV_H
int apsr_utf8_decode		(char *, const char *);	
#else
unsigned char *apsr_utf8_decode	(unsigned char *);
#endif
char * apsr_itoa		(int);

/* apsrlib-malloc.c  */
void * xmalloc                  (size_t);               
void * xrealloc                 (void *, size_t);       
void * xcalloc                  (size_t, size_t);       

/* apsrlib-generic.c */
struct ether_addr * apsr_get_hwinfo	(const char *, int);

/* apsrlib-endianess */
int apsr_endian_test		(void);

/* apsrlib-raw.c */
int apsr_open_raw_socket        (int, int);
int apsr_close_raw_socket       (int);
int apsr_write_raw_socket       (int, unsigned char *, int, in_addr_t, int, int );

char * apsr_get_time_formatted	(void);			/* apsrlib-time.c     */

/* apsrlib-sighandler.c */
int apsr_install_sig		(apsr_exit_callback);   
void apsr_check_uid		(void);			/* apsrlib-checkuid.c */

/* apsrlib-sockpacket.c */
struct apsr_link_layer * apsr_open_link_layer	(char *);
int apsr_close_link_layer	(struct apsr_link_layer *);
int apsr_write_link_layer	(struct apsr_link_layer *, char *,
		                 unsigned char *, int, int);
int apsr_read_link_layer	(struct apsr_link_layer *, int,
				 int, apsr_sniff_callback);

#if __OSTYPE_LINUX && HAVE_LIBBLUETOOTH
/* apsrlib-bluetooth.c */
int apsr_open_bluetooth_socket  (int);
int apsr_close_bluetooth_socket (int);
int apsr_read_bluetooth_socket  (int, int, int, apsr_read_bluetooth_callback);
#endif

/* apsrlib-ethernet.c */
int apsr_ethernet	(unsigned char *, unsigned char *, unsigned short,
			 const unsigned char *, int, unsigned char *);

int apsr_ethernet_snap  (unsigned int, unsigned int, unsigned int,
                         unsigned char *, unsigned short,
                         unsigned char *buf);

/* apsrlib-arp.c */
int apsr_arp	(unsigned short, unsigned short, unsigned char, 
		 unsigned char, unsigned short, unsigned char *, 
		 unsigned char *, unsigned char *, unsigned char *,
		 const unsigned char *, int,
		 unsigned char *);

/* apsrlib-ipv4.c */
int apsr_ipv4	(unsigned char, unsigned char, unsigned short, unsigned int,
		 unsigned short, unsigned short, unsigned char, unsigned char,
		 unsigned long, unsigned long, unsigned short,
		 unsigned char *, unsigned long, const unsigned char *,
		 int, unsigned char *);

/* apsrlib-ipv6.c */
int apsr_ipv6	(unsigned char, unsigned char, unsigned int, unsigned short,
		 unsigned char, unsigned char, struct in6_addr, struct in6_addr,
		 const unsigned char *, int, unsigned char *);

/* apsrlib-tcp.c */
int apsr_tcp	(unsigned short, unsigned short, unsigned long, unsigned long,
		 unsigned char, unsigned int, unsigned int, unsigned int,
		 unsigned short, unsigned short, unsigned short,
		 const unsigned char *, int, unsigned char *);

/* asprlib-udp.c */
int apsr_udp	(unsigned short, unsigned short, unsigned short, unsigned short,
		 const unsigned char *, int, unsigned char *);

/* asprlib-pppoe.c */
int apsr_pppoe  (unsigned char, unsigned char, unsigned char, unsigned short,                    unsigned short, const unsigned char *, int, unsigned char *);

/* asprlib-ppp.c */
int apsr_ppp  	(unsigned short, const unsigned char *, int, unsigned char *);

/* asprlib-ppp_lcp.c */
int apsr_ppp_lcp (unsigned char, unsigned char, unsigned short, unsigned char, unsigned char, const unsigned char *, int, unsigned char *);

/* apsrlib-ggp.c */
int apsr_ggp    (unsigned int, unsigned int, unsigned short,
                 const unsigned char *, int, unsigned char *);

/* apsrlib-cbt.c */
int apsr_cbt	(unsigned char, unsigned char, unsigned char, 
		 unsigned short, const unsigned char *, int, unsigned char *);

/* apsrlib-egp.c */
int apsr_egp    (unsigned int, unsigned int, unsigned int, unsigned int,
                 unsigned short, unsigned short, unsigned short,
                 const unsigned char *, int, unsigned char *);

/* apsrlib-ipauth.c */
int apsr_ipauth (unsigned int, unsigned int, unsigned short,
                 unsigned long, unsigned long, const unsigned char *,
                 int, unsigned char *);

/* apsrlib-ipesp.c */
int apsr_ipesp  (unsigned long, unsigned long, const unsigned char *,
                 int, unsigned char *);

/* apsrlib-igrp.c */
int apsr_igrp   (unsigned int, unsigned int, unsigned short, unsigned short,
                 unsigned short, unsigned short, unsigned short,
                 const unsigned char *, int, unsigned char *);

/* apsrlib-eigrp.c */
int apsr_eigrp  (unsigned char, unsigned char, unsigned short, unsigned long,
		 unsigned long, unsigned long, unsigned long,
		 const unsigned char *, int, unsigned char *);

/* apsrlib-igmpv2.c */
int apsr_igmpv2 (unsigned int, unsigned int, unsigned short,
                 unsigned long, const unsigned char *, int, unsigned char *);

/* apsrlib-ripng.c */
int apsr_ripng  (unsigned int, unsigned int, unsigned short,
                 const unsigned char *, int, unsigned char *);

/* apsrlib-rip.c */
int apsr_rip    (unsigned int, unsigned int, unsigned short, unsigned short,
                 unsigned short, unsigned long, unsigned long, unsigned long,
                 unsigned long, const unsigned char *, int, unsigned char *buf);

/* apsrlib-narp.c */
int apsr_narp	(unsigned int, unsigned int, unsigned short, unsigned int,
		 unsigned int, unsigned short, unsigned long, unsigned long,
		 unsigned int, const unsigned char *, int, unsigned char *);

/* apsrlib-gre.c */
int apsr_gre	(unsigned short, unsigned short, const unsigned char *,
		 int, unsigned char *);

/* apsrlib-ifmp.c */
int apsr_ifmp	(unsigned int, unsigned int, unsigned short,
		 const unsigned char *, int, unsigned char *);

/* apsrlib-pim.c */
int apsr_pim	(unsigned int, unsigned int, unsigned int, unsigned short,
		 const unsigned char *, int, unsigned char *);

/* apsrlib-irtp.c */
int apsr_irtp	(unsigned char, unsigned char, unsigned short, unsigned short,
		 unsigned short, const unsigned char *, int, unsigned char *);

/* apsrlib-cdp.c */
int apsr_cdp    (unsigned int, unsigned int, unsigned short,
		 const unsigned char *, int, unsigned char *);

/* apsrlib-sctp.c */
int apsr_sctp   (unsigned short, unsigned short, unsigned long, unsigned long,
		 const unsigned char *, int, unsigned char *);
 

/* apsrlib-cip.c */
int apsr_cip	(unsigned short int, unsigned short int, unsigned short int,
		 unsigned int, unsigned int, unsigned short int,
		 unsigned short int, unsigned char *, const unsigned char *,
		 int, unsigned char *);

/* apsrlib-icmpv4.c */
int apsr_icmpv4 (unsigned char, unsigned char, unsigned short int,
	         const unsigned char *, int, unsigned char *);
int apsr_icmpv4_echo (unsigned char, unsigned char, unsigned short int,
                 unsigned short, unsigned short, const unsigned char *,
  		 int, unsigned char *);
int apsr_icmpv4_timestamp (unsigned char, unsigned char, unsigned short int,
		 unsigned short, unsigned short, unsigned int, unsigned int,
		 unsigned int, const unsigned char *, int, unsigned char *);

/* apsrlib-in_cksum.c */
int in_cksum				(unsigned short *, int, int);

/* apsrlib-checksum.c */
int apsr_checksum			(unsigned char *, int, int);
int apsr_udpv4_checksum			(struct udp_pkt_header *);
int apsr_tcpv4_checksum                 (struct tcp_pkt_header *);

/* apsrlib-output.c */
int apsr_output				(char *, ...);
int apsr_stream_output			(FILE *, char *, ...);
int apsr_toggle_quiet			(void);
void apsr_set_quiet			(void);
void apsr_unset_quiet			(void);
int apsr_toggle_rquiet			(void);
void apsr_set_rquiet			(void);
void apsr_unset_rquiet			(void);

