#include "../include/apsrlib.h"

#if __OSTYPE_LINUX && HAVE_LIBBLUETOOTH
int apsr_open_bluetooth_socket (int dev_number) {
        struct sockaddr_hci addr;
        struct hci_filter flt;
	int fd, temp_opt;

	/* Open Socket */
	fd = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);

	if(fd <  0) {
	  #ifdef APSRLIB_DEBUG
	    ERROR("apsr_open_bluetooth_socket: socket failed!");
	  #endif
	
	  goto error;
	}

	/* Set temp_opt to 1 */
	temp_opt = 1;
	
	/* Set socket option data direction info */
        if (setsockopt(fd, SOL_HCI, HCI_DATA_DIR, &temp_opt, sizeof(temp_opt)) < 0) {
          #ifdef APSRLIB_DEBUG
            ERROR("apsr_open_bluetooth_socket: setsockopt(1) failed!");
          #endif

          goto error;
        }

	/* Re-Set temp_opt to 1 */
	temp_opt = 1;
	
	/* Set socket option time stamp */
        if (setsockopt(fd, SOL_HCI, HCI_TIME_STAMP, &temp_opt, sizeof(temp_opt)) < 0) {
          #ifdef APSRLIB_DEBUG
            ERROR("apsr_open_bluetooth_socket: setsockopt(2) failed!");
          #endif

          goto error;
        }

        /* Config filter */
        hci_filter_clear(&flt);
        hci_filter_all_ptypes(&flt);
        hci_filter_all_events(&flt);

	/* Set socket option HCI filter */
        if (setsockopt(fd, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
          #ifdef APSRLIB_DEBUG
            ERROR("apsr_open_bluetooth_socket: setsockopt(3) failed!");
          #endif

          goto error;	
        }

	/* Bind Socket to device */
        addr.hci_family = AF_BLUETOOTH;
        addr.hci_dev = dev_number;
        if (bind(fd, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
	  #ifdef APSRLIB_DEBUG
  	    ERROR("apsr_open_bluetooth_socket: bind failed!\n");
	  #endif  

	  goto error;
        }

	return(fd);

error:
	if(fd >= 0) close(fd);

	return(RET_FAILURE);
}


int apsr_close_bluetooth_socket (int fd) {
	if(close(fd) == 0) 
		return(RET_SUCCESS);
	else 
		return(RET_FAILURE);
}


int apsr_read_bluetooth_socket (int fd, int buf_size, int count, apsr_read_bluetooth_callback call_function) {
	struct cmsghdr *cmsg;
	struct msghdr msg;
	struct iovec iv;
	struct frame frm;
	unsigned char *ctrl, *buf;
	
	buf = xmalloc(buf_size);
	ctrl = xmalloc(100);
	frm.data = buf;
	
        memset(&msg, 0, sizeof(msg));

        while (1) {
		iv.iov_base = frm.data;
		iv.iov_len  = buf_size;
		
		/* Set msg struct */
                msg.msg_iov = &iv;
                msg.msg_iovlen = 1;
                msg.msg_control = ctrl;
                msg.msg_controllen = 100;

		if ((frm.data_len = recvmsg(fd, &msg, 0)) < 0) {
			#ifdef APSRLIB_DEBUG
			  INFO("apsr_read_bluetooth_socket: recvfrom failed");
			#endif
			apsr_close_bluetooth_socket(fd);
                        exit(EXIT_FAILURE);
	        }

  	       	/* Process control message */
		frm.in = 0;
		cmsg = CMSG_FIRSTHDR(&msg);
		while (cmsg) {
			switch (cmsg->cmsg_type) {
				case HCI_CMSG_DIR:
					frm.in = *((int *)CMSG_DATA(cmsg));
					break;
				case HCI_CMSG_TSTAMP:
				        frm.ts = *((struct timeval *)CMSG_DATA(cmsg));
				        break;
	                }
        	        cmsg = CMSG_NXTHDR(&msg, cmsg);
	        }
        	frm.ptr = frm.data;
	        frm.len = frm.data_len;
	
		/* start callback */	
		call_function(frm.ptr, frm.len);
	}	
	
        return(RET_SUCCESS);
}
#endif

