#include "../include/apsrlib-config.h" /* needed for apsr_htons */

/*
 * in_cksum --
 *      Checksum routine for Internet Protocol family headers (C Version)
 */

#include <netinet/in.h>

/* in_cksum : for calculating IP checksums with verification */
int in_cksum(unsigned short *addr, int len, int csum) {
        register int sum = csum;
        unsigned short answer = 0;
        register unsigned short *w = addr;
        register int nleft = len;

        /*
         * Our algorithm is simple, using a 32 bit accumulator (sum), we add
         * sequential 16 bit words to it, and at the end, fold back all the
         * carry bits from the top 16 bits into the lower 16 bits.
         */
        while (nleft > 1)  {
                sum += *w++;
                nleft -= 2;
        }

        /* mop up an odd byte, if necessary */
        if (nleft == 1) {
		sum += apsr_htons(*(unsigned char *)w<<8);
        }

        /* add back carry outs from top 16 bits to low 16 bits */
        sum = (sum >> 16) + (sum & 0xffff);     /* add hi 16 to low 16 */
        sum += (sum >> 16);                     /* add carry */
        answer = ~sum;                          /* truncate to 16 bits */
        return(answer);
}

int pseudo_in_cksum(unsigned short *addr, int len) {
	register int sum = 0;
	register unsigned short *w = addr;
	register int nleft = len;

	/*
	 * Our algorithm is simple, using a 32 bit accumulator (sum), we add
	 * sequential 16 bit words to it, and at the end, fold back all the
	 * carry bits from the top 16 bits into the lower 16 bits.
	 */
	
	while (nleft > 1)  {
		sum += *w++;
		nleft -= 2;
        }

        /* mop up an odd byte, if necessary */
        if (nleft == 1) {
		sum += apsr_htons(*(unsigned char *)w<<8);
        }

	return(sum);
}

