#include "../include/apsrlib-config.h"

/* Create a NARP (NBMA ARP) packet. */
int apsr_narp (unsigned int version, unsigned int hop, unsigned short sum,
	       unsigned int type, unsigned int code, unsigned short reserved,
	       unsigned long dst_ip, unsigned long src_ip, unsigned int length,
	       const unsigned char *payload, int payload_size,
	       unsigned char *buf)
{
	static struct narp_pkt_header narp_hdr;

	if(!buf) {
	  #ifdef APSRLIB_DEBUG
          ERROR("apsr_narp: !buf");
	  #endif
	  return(RET_FAILURE);
	}

	narp_hdr.version	= version;
	narp_hdr.hop		= hop;
	narp_hdr.sum		= apsr_htons(sum);
	narp_hdr.type		= type;
	narp_hdr.code		= code;
	narp_hdr.reserved	= apsr_htons(reserved);
	narp_hdr.dst_ip.s_addr	= dst_ip;
	narp_hdr.src_ip.s_addr	= src_ip;
	narp_hdr.length		= length;

	/* Check the payload size and copy it to the buffer. */
	if(payload && payload_size) {
	  memcpy(buf + APSR_NARP, payload, payload_size);
	}

	memcpy((unsigned char *)buf, (unsigned char *)&narp_hdr,
	       APSR_NARP_H);

	return(RET_SUCCESS);
}

