#include "../include/apsrserver-general.h"

int start_server(unsigned short port) {
	int sock_clt, reuse_addr = 1;
	pid_t child_pid;
	struct sockaddr_in servAddr, clientAddr;

	#ifndef socklen_t
	typedef unsigned int socklen_t;
	#endif


	socklen_t client_len = (socklen_t)sizeof(struct sockaddr_in);
        struct sigaction sa;
        sigset_t chld;
		
 	apsr_output("Starting APSR-Server.\n");
	
	if((sock_srv = socket(FAMILY, SOCK_STREAM, 0)) < 0) {
		apsr_stream_output((FILE *) stderr, "Error: Can't open server socket.\n");
		#ifdef APSRSERVER_DEBUG
		apsr_stream_output((FILE *) stderr, "Error code: %s.\n",
				strerror(errno));
		#endif
		exit(EXIT_FAILURE);
        }

        /* So we can re-bind to socket without TIME_WAIT problems */
        if(setsockopt(sock_srv, SOL_SOCKET, SO_REUSEADDR, &reuse_addr, sizeof(reuse_addr)) < 0) {
		apsr_stream_output((FILE *) stderr, "Error: Can't set socket options.\n");
		#ifdef APSRSERVER_DEBUG
		apsr_stream_output((FILE *) stderr, "Error code: %s.\n", 
				strerror(errno));
		#endif
	}
	
	memset(&servAddr, 0, sizeof(servAddr));
	servAddr.sin_family = FAMILY;
	servAddr.sin_addr.s_addr = apsr_htonl(INADDR_ANY);
	servAddr.sin_port = apsr_htons(port);

	if(bind(sock_srv,(struct sockaddr *) &servAddr, sizeof(servAddr)) < 0) {
		apsr_stream_output((FILE *) stderr, "Error: Can't bind server.\n");
		#ifdef APSRSERVER_DEBUG
		apsr_stream_output((FILE *) stderr, "Error code: %s.\n",
				strerror(errno));
		#endif
		exit(EXIT_FAILURE);
	}
	
	if(getsockname(sock_srv,(struct sockaddr *) &servAddr, &client_len) < 0) {
		apsr_stream_output((FILE *) stderr, "Error: Getsockname failed.\n");
                #ifdef APSRSERVER_DEBUG
		apsr_stream_output((FILE *) stderr, "Error code: %s.\n",
				strerror(errno));
                #endif
		exit(EXIT_FAILURE);
	}
		
	apsr_output("APSR-Server: Port number is %u.\n", port);

	if(listen(sock_srv, APSRSERVER_QUEUE) < 0) {
		apsr_stream_output((FILE *) stderr, "Error: Listen to socket failed.\n");
		#ifdef APSRSERVER_DEBUG
		apsr_stream_output((FILE *) stderr, "Error code: %s.\n",
				strerror(errno));
		#endif
		exit(EXIT_FAILURE);
	}

        /* Set up signal handler for childs */
        memset(&sa, 0, sizeof(sa));
        sa.sa_handler = sigchild_handler;
	
        if(sigaction(SIGCHLD, &sa, NULL) < 0) {
                #ifdef APSRSERVER_DEBUG
 		apsr_stream_output((FILE *) stderr, "Error: Sigaction call failed.\n");
		#endif
		exit(EXIT_FAILURE);
	}
					
        if(sigemptyset(&chld) < 0) {
                #ifdef APSRSERVER_DEBUG
		apsr_stream_output((FILE *) stderr, "Error: Sigemptyset call failed.\n");
	        #endif
		exit(EXIT_FAILURE);
        }
	
        if(sigaddset(&chld, SIGCHLD) < 0) {
                #ifdef APSRSERVER_DEBUG
		apsr_stream_output((FILE *) stderr, "Error: Sigaddset call failed.\n");
		#endif
	        exit(EXIT_FAILURE);
	}
						
restart:
	sock_clt = accept(sock_srv, (struct sockaddr *) &clientAddr,
			&client_len);
        if(sock_clt < 0 && errno == EINTR) {
	        goto restart;
        }
        else if(sock_clt < 0) {
		#ifdef APSRSERVER_DEBUG
	        apsr_stream_output((FILE *) stderr, "Error: Accept for client failed.\n");
		#endif
                exit(EXIT_FAILURE);
        }

	/* -|- TODO -|- Should be later moved into the forked server,	*/
	/* to avoid blocking while connect 				*/
	clients_count++;
	if(clients_count > APSRSERVER_MAX_CLIENTS) {
		apsr_output("APSR-Server: Connection from %s:%u denied, too many client connections.\n",
				inet_ntoa(clientAddr.sin_addr),
				apsr_ntohs(clientAddr.sin_port));
		clients_count--;
		if(close(sock_clt) < 0) {
			#ifdef APSRSERVER_DEBUG
                        apsr_stream_output((FILE *) stderr, "Error: close(sock_clt) failed.\n");
	                #endif
                }
		goto restart;
	}
	else {
		apsr_output("APSR-Server: Connection from %s:%u accepted, currently working %d/%d client(s).\n",
                                inet_ntoa(clientAddr.sin_addr),
                                apsr_ntohs(clientAddr.sin_port),
				clients_count,
				APSRSERVER_MAX_CLIENTS);
        }
        /* -|- TODO -|- */
	
	if((child_pid = fork()) < 0) {
		#ifdef APSRSERVER_DEBUG
		apsr_stream_output((FILE *) stderr, "Error: Fork for client failed.\n");
		#endif
		exit(EXIT_FAILURE);
	}
	else if(child_pid > 0) {
		apsr_output("APSR-Server: Connection from client %s:%u, handled by child with pid %d.\n", 
				inet_ntoa(clientAddr.sin_addr),
				apsr_ntohs(clientAddr.sin_port),
		                child_pid);
		if(close(sock_clt) < 0) {
			#ifdef APSRSERVER_DEBUG
			apsr_stream_output((FILE *) stderr, "Error: close(sock_clt) failed.\n");
			#endif
		}		
		goto restart;
	}
	else {
  		if(close(sock_srv) < 0) {
			#ifdef APSRSERVER_DEBUG
                        apsr_stream_output((FILE *) stderr, "Error: close(sock_srv) failed.\n");
	                #endif
                }
		return(aprecv_client_job(sock_clt, clientAddr));
 	}

	return(RET_SUCCESS);
}

int aprecv_client_job(int sock_clt, struct sockaddr_in clientAddr) {
	char apsrserver_buf[APSRSERVER_BUFFER_LENGTH];
	int command_length;
	#ifdef APSRSERVER_DEBUG
	unsigned int i;
	#endif
	unsigned short length;
	struct apsrserver_protocol_header *recv_header, send_header;

	#ifdef APSRSERVER_DEBUG
        char *client_ip=inet_ntoa(clientAddr.sin_addr);
	unsigned short client_port=apsr_ntohs(clientAddr.sin_port);
	#endif
	
	while(1) {
		memset(&apsrserver_buf, 0, APSRSERVER_BUFFER_LENGTH);
		if((command_length = recv(sock_clt, apsrserver_buf, 
				APSRSERVER_BUFFER_LENGTH, 0)) < 0) {
			#ifdef APSRSERVER_DEBUG
			apsr_stream_output((FILE *) stderr, "Error: Receive from aprecv client failed.\n");
			#endif
			exit(EXIT_FAILURE);
		}

		recv_header = (struct apsrserver_protocol_header *)apsrserver_buf;
		length = apsr_ntohs(recv_header->length);

  		#ifdef APSRSERVER_DEBUG
                apsr_output("APSR-Server(%s:%u): R<- [Pkt] V %d, ID %d, Cmd %d, Code %d, Len %d.\n",  
				client_ip, client_port,
				recv_header->version,
				apsr_ntohs(recv_header->id),
				recv_header->command,
				recv_header->code,
				length);
		if(length > sizeof(struct apsrserver_protocol_header)) { 
			apsr_output("Data: \"");
	                for(i = 0; i < length; i++) {
				if(isprint(apsrserver_buf[i]))
					apsr_output("%c", apsrserver_buf[i]);
	                        else
        	                        apsr_output(".");
			}
	        apsr_output("\".\n");
		}
		#endif		
		
		if(recv_header->version != APSR_APSRSERVER_VERSION) {
	          #ifdef APSRSERVER_DEBUG
		  apsr_output("APSR-Server(%s:%u): Incompatibel packet version, ignore.\n",
				  client_ip, client_port);
		  #endif
		}
	        else {
  		  switch(recv_header->command) {
		    case APSR_APSRSERVER_REQ_LOGIN:
		      switch(recv_header->code) {
		        case APSR_APSRSERVER_REQ_LOGIN_COD_LOGIN:
			  #ifdef APSRSERVER_DEBUG
  			  apsr_output("APSR-Server(%s:%u): Client request for login.\n",
					client_ip, client_port);
			  #endif

  			  send_header.version = APSR_APSRSERVER_VERSION;
			  send_header.id      = apsr_htons(2342); /* TEST */
			  send_header.command = APSR_APSRSERVER_REP_LOGIN;
			  send_header.code    = APSR_APSRSERVER_REP_LOGIN_COD_OK;
			  send_header.length  = apsr_htons(sizeof(struct apsrserver_protocol_header));
			  
			  #ifdef APSRSERVER_DEBUG
			  apsr_output("APSR-Server(%s:%u): Send reply for login.\n",
					  client_ip, client_port);
	                  apsr_output("APSR-Server(%s:%u): S-> [Pkt] V %d, ID %d, Cmd %d, Code %d, Len %d.\n",
			        client_ip, client_port,
				send_header.version,
				apsr_ntohs(send_header.id),
				send_header.command,
				send_header.code,
				apsr_ntohs(send_header.length));
			  #endif
      			  if(send(sock_clt,  
				(unsigned char *)&send_header,
                                sizeof(send_header), 0) < 0) {
		            #ifdef APSRSERVER_DEBUG
		            apsr_stream_output((FILE *) stderr, "Error: Sending(reply login) failure.\n");
                            #endif
                            exit(EXIT_FAILURE);
                          }
			  break;
	                case APSR_APSRSERVER_REQ_LOGIN_COD_LOGOUT:
	                  #ifdef APSRSERVER_DEBUG
			  apsr_output("APSR-Server(%s:%u): Client request for logout.\n",
					client_ip, client_port);
			  #endif
			  goto end_client;
			  /* never reached */
		          break;
		        default:
		          break;	
	              }
		      break;
                    case APSR_APSRSERVER_GOT_PACKET:
                      switch(recv_header->code) {
                        case APSR_APSRSERVER_GOT_PACKET_COD_PLAIN:
                          #ifdef APSRSERVER_DEBUG
	                  apsr_output("APSR-Server(%s:%u): Client got a packet.\n",
      				  client_ip, client_port);
                          #endif
                          break;
		      }
		      break;
  		    default:
		      break;
		  }
               }
		
	}
		
end_client:
	
	return(RET_SUCCESS);
}

void clean_exit (int sig) {
	/* Don't work @ the moment i think, 				*/
	/* should close server socket and all forked child processes 	*/
	if(shutdown(sock_srv, 2) < 0) {
		#ifdef APSRSERVER_DEBUG
                apsr_stream_output((FILE *) stderr, "Error: Can't shutdown all processes(server socket: %d, code %d).\n", 
				sock_srv, errno);
                #endif
		exit(EXIT_FAILURE);
        }

	exit(EXIT_SUCCESS);
}
	
void sigchild_handler(int sig) {
	pid_t pid;

	clients_count--;
	if((pid = wait(NULL)) < 0) {
		apsr_stream_output((FILE *) stderr, "Error: Wait failed.\n");
		exit(EXIT_FAILURE);
        }
		
	apsr_output("APSR-Server: Child process with pid %d exited, now %d/%d client(s).\n", 
      			pid, clients_count, APSRSERVER_MAX_CLIENTS);
}
