#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.02"
#define MODULE_NAME		"dhcp"
#define MODULE_DESCRIPTION	"dhcp (dynamic host configuration protocol) sniffer"
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	67
#define MODULE_IDENT_ID2	68

#define MODULE_FILTER		"udp and port 67 or port 68"

#define APSR_DHCP_REQUEST	1
#define APSR_DHCP_REPLY		2

/* TODO: DHCP options */

struct mod_dhcp_header {
	unsigned int dhcp_op:8;	/* packet opcode type */
	unsigned int dhcp_htype:8;	/* hardware addr type */
	unsigned int dhcp_hlen:8;	/* hardware addr length */
	unsigned int dhcp_hops:8;	/* gateway hops */
	unsigned int dhcp_xid;		/* transaction ID */
	unsigned short dhcp_secs;	/* seconds since client began */
					/* address address acquisition or */
					/* renewal process */
	unsigned short dhcp_flags;	/* flags: 0x8000 is broadcast */
	struct in_addr  dhcp_ciaddr;	/* client IP address */
	struct in_addr  dhcp_yiaddr;	/* 'your' IP address */
	struct in_addr  dhcp_siaddr;	/* server IP address */
	struct in_addr  dhcp_giaddr;	/* gateway IP address */
	unsigned char dhcp_chaddr[16];	/* client hardware address */
	unsigned char dhcp_sname[64];	/* server host name */
	unsigned char dhcp_file[128];	/* boot file name */
	unsigned char dhcp_vend[64];	/* vendor-specific area */
};


struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER;
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
					
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
	const struct mod_dhcp_header *dhcp_header;

 	dhcp_header = (const struct mod_dhcp_header *) buf;
	apsr_output("- DHCP header -\n");

	apsr_output("  Opcode type: %u ", dhcp_header->dhcp_op);
	if(dhcp_header->dhcp_op	== APSR_DHCP_REQUEST)
 		apsr_output("(DHCP request)\n");
	else if(dhcp_header->dhcp_op == APSR_DHCP_REPLY)
		apsr_output("(DHCP reply)\n");
	else
		apsr_output("(unknown DHCP opcode!)\n");
	
	apsr_output("  Hardware type: %u ", dhcp_header->dhcp_htype);
	switch(dhcp_header->dhcp_htype) {
		case APSR_ARPHRD_NETROM:
		  apsr_output("(NETROM)\n");
		  break;
		case APSR_ARPHRD_ETHER:
		  apsr_output("(Ethernet 10/100Mbps)\n");
		  break;
		case APSR_ARPHRD_EETHER:
		  apsr_output("(Experimental Ethernet)\n");
		  break;
		case APSR_ARPHRD_AX25:
		  apsr_output("(AX.25 Level 2)\n");
		  break;
		case APSR_ARPHRD_PRONET:
		  apsr_output("(PROnet token ring)\n");
		  break;
		case APSR_ARPHRD_CHAOS:
		  apsr_output("(Chaosnet)\n");
		  break;
		case APSR_ARPHRD_IEEE802:
		  apsr_output("(IEEE 802.2 Ethernet/TR/TB)\n");
		  break;
		case APSR_ARPHRD_ARCNET:
		  apsr_output("(ARCNET)\n");
		  break;
		case APSR_ARPHRD_APPLETLK:
		  apsr_output("(Appletalk)\n");
		  break;
		case APSR_ARPHRD_DLCI:
		  apsr_output("(Frame Relay DLCI)\n");
		  break;
		case APSR_ARPHRD_ATM:
		  apsr_output("(ATM)\n");
		  break;
		case APSR_ARPHRD_METRICOM:
		  apsr_output("(Metricom STRIP)\n");
		  break;
		default:
		  apsr_output("(unknown type)\n");
		  break;
	};

	apsr_output("  Hardware length: %u\n", dhcp_header->dhcp_hlen);
	apsr_output("  Gateway hops: %u\n", dhcp_header->dhcp_hops);
	apsr_output("  Transaction ID: 0x%08x\n", apsr_ntohl(dhcp_header->dhcp_xid));
	apsr_output("  Seconds since client began address acquisition/renewal process: %u\n", apsr_ntohs(dhcp_header->dhcp_secs));
	
	apsr_output("  Flags: %u ", apsr_ntohs(dhcp_header->dhcp_flags));
	if(dhcp_header->dhcp_flags == 0x8000)
		apsr_output("(broadcast)\n");
	else
		apsr_output("(unused)\n");

	if(dhcp_header->dhcp_op == APSR_DHCP_REQUEST)
		apsr_output("  Client IP addr: %s\n", inet_ntoa(dhcp_header->dhcp_ciaddr));
	else if(dhcp_header->dhcp_op == APSR_DHCP_REPLY)
		apsr_output("  Server IP addr: %s\n", inet_ntoa(dhcp_header->dhcp_siaddr));
		
	if(dhcp_header->dhcp_ciaddr.s_addr == 0)
		apsr_output("  Yclient IP addr: %s\n", inet_ntoa(dhcp_header->dhcp_yiaddr));

	apsr_output("  Gateway IP addr: %s\n", inet_ntoa(dhcp_header->dhcp_giaddr));
	apsr_output("  Client hardware addr: %s", ether2str((unsigned char *)dhcp_header->dhcp_chaddr));
	if(verbose)
		apsr_output(" (%s)\n", apsr_resolve_mac((unsigned char *)dhcp_header->dhcp_chaddr));
	else 
		apsr_output("\n");
	    
	if(*dhcp_header->dhcp_sname)
		apsr_output("\n  Server hostname: %s\n", dhcp_header->dhcp_sname);
	
	if(*dhcp_header->dhcp_file)
		apsr_output("  Boot file name: \"%s\"\n", dhcp_header->dhcp_file);
	
	if(*dhcp_header->dhcp_vend)
		apsr_output("  Vendor spec. exts: %s\n", "(not implemented)");
	
	#ifdef MODULE_DEBUG
	apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
		       family, protocol_number, id1, id2);
	#endif
	  
	return(RET_SUCCESS);
}

