#include <apsrlib/apsrlib.h>

/* TODO: RR EDNS0/EDNS0_ELT detection */
/* TODO: date like TTL printing */
/* TODO: Rdata printing */
/* TODO: move switch/case stuff into structs */
/* TODO: more types */

#define MODULE_VERSION		"0.08"
#define MODULE_NAME		"dns"
#define MODULE_DESCRIPTION	"Example DNS protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_PN2	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	53	
#define MODULE_IDENT_ID2	53

#define MODULE_FILTER		"(tcp or udp) and port 53"

#define APSR_DNS_QR	0x80
#define APSR_DNS_AA	0x04
#define APSR_DNS_TC	0x02
#define APSR_DNS_RD	0x01
#define APSR_DNS_RA 	0x80

#define APSR_DNS_OPCODE_QUERY			0		/* rfc 1035 */
#define	APSR_DNS_OPCODE_INV_QUERY		1		/* rfc 1035 */
#define	APSR_DNS_OPCODE_STATUS			2		/* rfc 1035 */
#define APSR_DNS_OPCODE_RESERVED		3		
#define APSR_DNS_OPCODE_NOTIFY			4		/* rfc 1996 */
#define APSR_DNS_OPCODE_UPDATE			5		/* rfc 2136 */
#define	APSR_DNS_OPCODE_UPDATE_RECORD		9
#define APSR_DNS_OPCODE_DELETE_S_RECORD		10
#define	APSR_DNS_OPCODE_DELETE_A_RECORD		11
#define	APSR_DNS_OPCODE_MODIFY_S_RECORD		12
#define APSR_DNS_OPCODE_MODIFY_A_RECORD		13
#define	APSR_DNS_OPCODE_INIT_ZONE		14
#define	APSR_DNS_OPCODE_INCR_ZONE		15

#define APSR_DNS_RCODE_NOERROR			0		/* rfc 1035 */
#define APSR_DNS_RCODE_FORMAT_ERROR		1		/* rfc 1035 */
#define APSR_DNS_RCODE_SERVER_FAILURE		2		/* rfc 1035 */
#define APSR_DNS_RCODE_NOT_EXISTING_DOM		3		/* rfc 1035 */
#define APSR_DNS_RCODE_NOT_IMPLEMENTED		4		/* rfc 1035 */
#define APSR_DNS_RCODE_QUERY_REFUSED		5		/* rfc 1035 */
#define APSR_DNS_RCODE_YXDOMAIN			6		/* rfc 2136 */
#define APSR_DNS_RCODE_YXRRSET			7		/* rfc 2136 */
#define APSR_DNS_RCODE_NXRRSET			8		/* rfc 2136 */
#define APSR_DNS_RCODE_NOTAUTH			9		/* rfc 2136 */
#define APSR_DNS_RCODE_NOTZONE			10		/* rfc 2136 */
/* #define APSR_DNS_RCODE_RESERVED 11 - 14 */
#define APSR_DNS_RCODE_NO_CHANGE		15		/* ??? */
#define APSR_DNS_RCODE_BADVERS			16		/* rfc 2761 */
#define APSR_DNS_RCODE_BADKEY			17		/* rfc 2845 */
#define APSR_DNS_RCODE_BADTIME			18		/* rfc 2845 */
#define APSR_DNS_RCODE_BADMODE			19		/* rfc 2930 */
#define APSR_DNS_RCODE_BADNAME			20		/* rfc 2930 */
#define APSR_DNS_RCODE_BADALG			21		/* rfc 2930 */

#define APSR_DNS_TYPE_A		1		/* IPv4 address */
#define APSR_DNS_TYPE_NS	2		/* Authoritative server */
#define APSR_DNS_TYPE_MD	3		/* Mail destination */
#define APSR_DNS_TYPE_MF	4		/* Mail forwarder */
#define APSR_DNS_TYPE_CNAME	5		/* Cannoncial name */
#define APSR_DNS_TYPE_SOA	6 		/* Start of authority zone */
#define APSR_DNS_TYPE_MB	7		/* Mailbox domain member */
#define APSR_DNS_TYPE_MG	8		/* Mail groub member */
#define APSR_DNS_TYPE_MR	9		/* Mail rename name */
#define APSR_DNS_TYPE_NULL	10		/* Null resource record */
#define APSR_DNS_TYPE_WKS	11		/* Well known service */
#define APSR_DNS_TYPE_PTR	12		/* Domain name pointer */
#define APSR_DNS_TYPE_HINFO	13		/* Host information */
#define APSR_DNS_TYPE_MINFO	14		/* Mailbox or mail list info */
#define APSR_DNS_TYPE_MX	15		/* Mail routing information */
#define APSR_DNS_TYPE_TXT	16		/* Text */
#define APSR_DNS_TYPE_RP	17		/* Responsible p. rfc 1183 */
#define APSR_DNS_TYPE_AFSDB	18		/* AFS DB location rfc 1183 */
#define APSR_DNS_TYPE_X25	19		/* X.25 call. addr. rfc 1183 */
#define APSR_DNS_TYPE_ISDN	20		/* ISDN address rfc 1183 */
#define APSR_DNS_TYPE_RT	21		/* Route through rfc 1183 */
#define APSR_DNS_TYPE_NSAP	22		/* NSAP address */
#define APSR_DNS_TYPE_NSAP_PTR	23		/* reverse lookup for NSAP */
#define APSR_DNS_TYPE_SIG	24		/* Security signature */
#define APSR_DNS_TYPE_KEY	25		/* Security key */
#define APSR_DNS_TYPE_PX	26		/* X.400 mail mapping */
#define APSR_DNS_TYPE_GPOS	27 		/* geographical position */
#define APSR_DNS_TYPE_AAAA	28		/* IPv6 address */
#define APSR_DNS_TYPE_LOC	29              /* Location Information */
#define APSR_DNS_TYPE_NXT	30              /* Next Valid Name in Zone */
#define APSR_DNS_TYPE_EID	31              /* Endpoint identifier */
#define APSR_DNS_TYPE_NIMLOC	32              /* Nimrod locator */
#define APSR_DNS_TYPE_SRV	33              /* Server selection */
#define APSR_DNS_TYPE_ATMA	34              /* ATM Address */
#define APSR_DNS_TYPE_NAPTR	35              /* Naming Authority Pointer */
#define APSR_DNS_TYPE_A6	38              /* IPv6 address */
#define APSR_DNS_TYPE_DNAME	39              /* non-terminal redirection */
#define APSR_DNS_TYPE_OPT	41              /* EDNS0 option (meta-RR) */
#define APSR_DNS_TYPE_UINFO	100             /* User(Finger) information */
#define APSR_DNS_TYPE_UID	101             /* User ID */
#define APSR_DNS_TYPE_GID	102             /* Group ID */
#define APSR_DNS_TYPE_UNSPEC	103             /* Unspecified format (binary data) */
#define APSR_DNS_TYPE_TKEY	249             /* Transaction Key (RFC 2930) */
#define APSR_DNS_TYPE_TSIG	250             /* Transaction Signature (RFC 2845) */
#define APSR_DNS_TYPE_ANY	255		/* Any */
#define APSR_DNS_TYPE_WINS	65281		/* Microsoft's WINS RR */
#define APSR_DNS_TYPE_WINS_R	65282		/* Microsoft's WINS-R RR */

#define APSR_DNS_CLASS_RES	0		/* Reserved */
#define APSR_DNS_CLASS_IN	1		/* Internet, rfc 1035 */
#define APSR_DNS_CLASS_CSNET	2		/* CSNET (obsolete) */
#define APSR_DNS_CLASS_CH	3		/* Chaos, rfc 1035 */
#define APSR_DNS_CLASS_HS	4		/* Hesiod, rfc 1035 */
#define APSR_DNS_CLASS_NONE	254             /* None rfc 2136 */
#define APSR_DNS_CLASS_ANY	255             /* Any rfc 1035 */


struct mod_dns_header {
	unsigned short id;
	unsigned char parameter[1];
	unsigned short number_questions;
	unsigned short number_answers_rr;
	unsigned short number_authority_rr;
	unsigned short number_additional_rr;
};

struct mod_dns_rr_header {
	unsigned short type;
	unsigned short class;
	unsigned int ttl;
	unsigned short data_length;
};

struct mod_dns_query_header {
	unsigned short type;
	unsigned short class;
};	

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = MODULE_IDENT_PN2;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
	      
	return(module_info);
}

extern char verbose;
int print_query(unsigned char *);
int print_rr(unsigned char *);
void verbose_type(unsigned short);
void verbose_class(unsigned short);

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_dns_header *dns_header;
  unsigned short i=0, ret=0;
  unsigned short num_questions, num_answers, num_authority, num_additional;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    i = 1;
	    dns_header = (struct mod_dns_header *)buf;
	    apsr_output("- DNS Header -\n");
	    apsr_output("  Identification: %d\n", apsr_ntohs(dns_header->id));
	    apsr_output("  Parameter: [");
	    if(APSR_DNS_QR & dns_header->parameter[0]) {
		    if(verbose)
			    apsr_output("Query/Response ");
		    else
			    apsr_output("qr ");
	    }
	    if(APSR_DNS_AA & dns_header->parameter[0]) {
		    if(verbose)
			    apsr_output("Authoriative-Answer ");
		    else
			    apsr_output("aa ");
	    }
	    if(APSR_DNS_TC & dns_header->parameter[0]) {
		    if(verbose)
			    apsr_output("Truncated ");
		    else
			    apsr_output("tc ");
	    }
	    if(APSR_DNS_RD & dns_header->parameter[0]) {
		    if(verbose)
			    apsr_output("Recursion-Desired ");
		    else
			    apsr_output("rd ");
	    }
	    if(APSR_DNS_RA & dns_header->parameter[1]) {
		    if(verbose)
			    apsr_output("Recursion-Available ");
		    else
			    apsr_output("ra ");
	    }
	    apsr_output("] ");
	    switch((dns_header->parameter[0] >> 3) & 0xF) {
		    case APSR_DNS_OPCODE_QUERY:
			   apsr_output("QUERY | ");
		    	   break;
		    case APSR_DNS_OPCODE_INV_QUERY:
			   apsr_output("INVERSE QUERY | ");
			   break;
		    case APSR_DNS_OPCODE_STATUS:
			   apsr_output("STATUS | ");
			   break;
		    case APSR_DNS_OPCODE_RESERVED:
			   apsr_output("RESERVED | ");
			   break;
	            case APSR_DNS_OPCODE_NOTIFY:
			   apsr_output("NOTIFY | ");
			   break;
	            case APSR_DNS_OPCODE_UPDATE:
			   apsr_output("UPDATE | ");
			   break;
		    case APSR_DNS_OPCODE_UPDATE_RECORD:
			   apsr_output("UPDATE RECORD | ");
			   break;
		    case APSR_DNS_OPCODE_DELETE_S_RECORD:
			   apsr_output("DELETE SPECIFIC RESOURCE RECORD | ");
			   break;
		    case APSR_DNS_OPCODE_DELETE_A_RECORD:
			   apsr_output("DELETE ALL RESOURCE RECORDS | ");
			   break;
		    case APSR_DNS_OPCODE_MODIFY_S_RECORD:
			   apsr_output("MODIFY SPECIFIC RESOURCE RECORD | ");
			   break;
		    case APSR_DNS_OPCODE_MODIFY_A_RECORD:
			   apsr_output("MODIFY ALL RESOURCE RECORDS | ");
			   break;
		    case APSR_DNS_OPCODE_INIT_ZONE:
			   apsr_output("INITIAL ZONE TRANSFER | ");
			   break;
		    case APSR_DNS_OPCODE_INCR_ZONE:
			   apsr_output("INCREMENTAL ZONE REFRESH | ");
			   break;
		    default:
			   /* Unknown */
			   break;
	    }
	    switch(dns_header->parameter[1] & 0xF) {
		    case APSR_DNS_RCODE_NOERROR:
			    apsr_output("NOERROR ");
			    break;
	            case APSR_DNS_RCODE_FORMAT_ERROR:
	    		    apsr_output("FORMAT ERROR ");
			    break;
	            case APSR_DNS_RCODE_SERVER_FAILURE:
			    apsr_output("SERVER FAILURE ");
			    break;
	            case APSR_DNS_RCODE_NOT_EXISTING_DOM:
			    apsr_output("NOT EXISTING DOMAIN "); 
			    break;
		    case APSR_DNS_RCODE_NOT_IMPLEMENTED:
			    apsr_output("NOT IMPLEMENTED ");
			    break;
	            case APSR_DNS_RCODE_QUERY_REFUSED:
			    apsr_output("QUERY REFUSED ");
			    break;
	            case APSR_DNS_RCODE_YXDOMAIN:
	    		    apsr_output("YXDOMAIN ");
			    break;
	            case APSR_DNS_RCODE_YXRRSET:
	                    apsr_output("YXRRSET ");
	                    break;
	            case APSR_DNS_RCODE_NXRRSET:
	                    apsr_output("NXRRSET ");
	                    break;
	            case APSR_DNS_RCODE_NOTAUTH:
	                    apsr_output("NOTAUTH ");
			    break;
		    case APSR_DNS_RCODE_NOTZONE:
			    apsr_output("NOTZONE ");
			    break;
		    case APSR_DNS_RCODE_NO_CHANGE:
			    apsr_output("NO CHANGE ");
			    break;
		    case APSR_DNS_RCODE_BADVERS:
			    apsr_output("BADVERS ");
			    break;
	            case APSR_DNS_RCODE_BADKEY:
	  		    apsr_output("BADKEY ");
	                    break;
	            case APSR_DNS_RCODE_BADTIME:
	                    apsr_output("BADTIME ");
	                    break;
	            case APSR_DNS_RCODE_BADMODE:
	                    apsr_output("BADMODE ");
	                    break;
		    case APSR_DNS_RCODE_BADNAME:
	                    apsr_output("BADNAME ");
	                    break;
	            case APSR_DNS_RCODE_BADALG:
	                    apsr_output("BADALG ");
	                    break;		    
		    default:
			    /* Unknown */
			    break;
            }
	    num_questions = apsr_ntohs(dns_header->number_questions);
	    num_answers   = apsr_ntohs(dns_header->number_answers_rr);
	    num_authority = apsr_ntohs(dns_header->number_authority_rr);
	    num_additional = apsr_ntohs(dns_header->number_additional_rr);
	    apsr_output("\n  Number of Questions: %d\n", num_questions);
	    apsr_output("  Number of Answers(RR): %d\n", num_answers);
	    apsr_output("  Number of Authority(RR): %d\n", num_authority);
	    apsr_output("  Number of Additional(RR): %d\n", num_additional);
	    for(i = 1; i <= num_questions; i++) {
		    apsr_output("  - Question(%d) -\n", i);
		    ret += print_query(buf+12+ret);
	    }
	    for(i = 1; i <= num_answers; i++) {
		    apsr_output("  - Answer(%d) -\n", i);
		    ret += print_rr(buf+12+ret);		
            }
	    for(i = 1; i <= num_authority; i++) {
		    apsr_output("  - Authority(%d) -\n", i);
		    ret += print_rr(buf+12+ret);
	    }
	    for(i = 1; i <= num_additional; i++) {
		    apsr_output("  - Additional(%d) -\n", i);
		    ret += print_rr(buf+12+ret);
            }
    }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}

int print_rr(unsigned char *buf) {
  unsigned short i, c, count_c, len, rrtype, rrclass, data_length;
  unsigned int ttl;
  struct mod_dns_rr_header *rr_header;

  apsr_output("    Name: \"");
  count_c = 1;
  i = 1;
  c = buf[0];
  len = strlen((char *) buf);
  while(i < len) {
	  if(i == (c+count_c)) {
		  count_c += 1;
		  c += buf[i];
		  apsr_output(".");
	  }
	  else {
		  if(isprint(buf[i]))
			  apsr_output("%c", buf[i]);
                  else
			  apsr_output(".");
	  }
          i++;
  }
  rr_header = (struct mod_dns_rr_header *)(buf+len);
  rrtype = apsr_ntohs(rr_header->type);
  rrclass = apsr_ntohs(rr_header->class);
  ttl = apsr_ntohl(rr_header->ttl);
  data_length = apsr_ntohs(rr_header->data_length);
  apsr_output("\"\n    Type: %u", rrtype);
  if(verbose)
	          verbose_type(rrtype);
  apsr_output("\n    Class: %u", rrclass);
  if(verbose)
	          verbose_class(rrclass);
  apsr_output("\n    TTL: %u", ttl);
  apsr_output("\n    Data Length: %u", data_length);
  apsr_output("\n");
  
  return(len+10+data_length);
}  
	  
int print_query(unsigned char *buf) {
  unsigned short i, c, count_c, len, qtype, qclass;
  struct mod_dns_query_header *q_header;
  
  apsr_output("    Query Name: \"");
  count_c = 1;
  i = 1;
  c = buf[0];
  len = strlen((char *) buf);
  while(i < len) {
	  if(i == (c+count_c)) {
		  count_c += 1;
		  c += buf[i];
		  apsr_output(".");
	  }
          else {
		  if(isprint(buf[i])) 
			  apsr_output("%c", buf[i]);
		  else
			  apsr_output(".");
	  }
	  i++;
  }
  q_header = (struct mod_dns_query_header *)(buf+len+1);
  qtype = apsr_ntohs(q_header->type);
  qclass = apsr_ntohs(q_header->class);
  apsr_output("\"\n    Type: %u", qtype);
  if(verbose) 
	  verbose_type(qtype);
  apsr_output("\n    Class: %u", qclass);
  if(verbose) 
	  verbose_class(qclass);
  apsr_output("\n");
  
  return(len+4);		
}
  
 
void verbose_type(unsigned short type) {
  switch(type) {
		  case APSR_DNS_TYPE_A:
		    apsr_output(" (IPv4 address)");
		    break;
		  case APSR_DNS_TYPE_NS:
		    apsr_output(" (Authoritative server)");
		    break;
		  case APSR_DNS_TYPE_MD:
		    apsr_output(" (Mail destination)");
		    break;
		  case APSR_DNS_TYPE_MF:
		    apsr_output(" (Mail forwarder)");
		    break;
		  case APSR_DNS_TYPE_CNAME:
		    apsr_output(" (Cannonical name)");
		    break;
		  case APSR_DNS_TYPE_SOA:
		    apsr_output(" (Start of authority zone)");
		    break;
		  case APSR_DNS_TYPE_MB:
		    apsr_output(" (Mailbox domain name)");
		    break;
		  case APSR_DNS_TYPE_MG:
		    apsr_output(" (Mail group member)");
		    break;
		  case APSR_DNS_TYPE_MR:
		    apsr_output(" (Mail rename name)");
		    break;
		  case APSR_DNS_TYPE_NULL:
		    apsr_output(" (Null resource record)");
		    break;
		  case APSR_DNS_TYPE_WKS:
		    apsr_output(" (Well known service)");
		    break;
		  case APSR_DNS_TYPE_PTR:
		    apsr_output(" (Domain name pointer)");
		    break;
		  case APSR_DNS_TYPE_HINFO:
		    apsr_output(" (Host information)");
		    break;
		  case APSR_DNS_TYPE_MINFO:
		    apsr_output(" (Mailbox or mail list info)");
		    break;
		  case APSR_DNS_TYPE_MX:
		    apsr_output(" (Mail routing information)");
		    break;
		  case APSR_DNS_TYPE_TXT:
		    apsr_output(" (Text string)");
		    break;
		  case APSR_DNS_TYPE_RP:
		    apsr_output(" (Responsible person)");
		    break;
		  case APSR_DNS_TYPE_AFSDB:
		    apsr_output(" (AFS DB location)");
		    break;
		  case APSR_DNS_TYPE_X25:
		    apsr_output(" (X.25 calling address)");
		    break;
		  case APSR_DNS_TYPE_ISDN:
		    apsr_output(" (ISDN address)");
		    break;
		  case APSR_DNS_TYPE_RT:
		    apsr_output(" (Route through)");
		    break;
		  case APSR_DNS_TYPE_NSAP:
		    apsr_output(" (NSAP address)");
		    break;
		  case APSR_DNS_TYPE_NSAP_PTR:
		    apsr_output(" (reverse lookup for NSAP)");
		    break;
		  case APSR_DNS_TYPE_SIG:
		    apsr_output(" (Security signature)");
		    break;
		  case APSR_DNS_TYPE_KEY:
		    apsr_output(" (Security key)");
		    break;
		  case APSR_DNS_TYPE_PX:
		    apsr_output(" (X.400 mail mapping)");
		    break;
		  case APSR_DNS_TYPE_GPOS:
		    apsr_output(" (geographical position)");
		    break;
		  case APSR_DNS_TYPE_AAAA:
		  case APSR_DNS_TYPE_A6:
		    apsr_output(" (IPv6 address)");
		    break;
                  case APSR_DNS_TYPE_LOC:
                    apsr_output(" (Location Information)");
                    break;
                  case APSR_DNS_TYPE_NXT:
                    apsr_output(" (Next Valid Name in Zone)");
                    break;
                  case APSR_DNS_TYPE_EID:
                    apsr_output(" (Endpoint identifier)");
                    break;
                  case APSR_DNS_TYPE_NIMLOC:
                    apsr_output(" (Nimrod locator)");
                    break;
                  case APSR_DNS_TYPE_SRV:
                    apsr_output(" (Server selection)");
                    break;
                  case APSR_DNS_TYPE_ATMA:
                    apsr_output(" (ATM Address)");
                    break;
                  case APSR_DNS_TYPE_NAPTR:
                    apsr_output(" (Naming Authority Pointer)");
                    break;
                  case APSR_DNS_TYPE_DNAME:
                    apsr_output(" (non-terminal redirection)");
                    break;
                  case APSR_DNS_TYPE_OPT:
                    apsr_output(" (EDNS0 option/meta-RR)");
                    break;
		  case APSR_DNS_TYPE_UINFO:
		    apsr_output(" (User(Finger) information)");
		    break;
		  case APSR_DNS_TYPE_UID:
		    apsr_output(" (User ID)");
		    break;
		  case APSR_DNS_TYPE_GID:
		    apsr_output(" (Group ID)");
		    break;
		  case APSR_DNS_TYPE_UNSPEC:
		    apsr_output(" (Unspecified format)");
		    break;
		  case APSR_DNS_TYPE_TKEY:
		    apsr_output(" (Transaction Key)");
		    break;
		  case APSR_DNS_TYPE_TSIG:
		    apsr_output(" (Transaction Signature)");
		    break;
		  case APSR_DNS_TYPE_ANY:
		    apsr_output(" (Any)");
		    break;
		  case APSR_DNS_TYPE_WINS:
		    apsr_output(" (Microsoft's WINS RR)");
		    break;
		  case APSR_DNS_TYPE_WINS_R:
		    apsr_output(" (Microsoft's WINS-R RR)");
		    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
  }
}

void verbose_class(unsigned short class) {
  switch(class) {
		  case APSR_DNS_CLASS_RES:
	            apsr_output(" (Reserved)");
		    break;
		  case APSR_DNS_CLASS_IN:
		    apsr_output(" (Internet)");
		    break;
		  case APSR_DNS_CLASS_CSNET:
		    apsr_output(" (CSNET)");
		    break;
		  case APSR_DNS_CLASS_CH:
		    apsr_output(" (Chaos)");
		    break;
		  case APSR_DNS_CLASS_HS:
		    apsr_output(" (Hesiod)");
		    break;
		  case APSR_DNS_CLASS_NONE:
		    apsr_output(" (None)");
		    break;
		  case APSR_DNS_CLASS_ANY:
		    apsr_output(" (Any)");
		    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
  }
}
