#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.02"
#define MODULE_NAME		"isakmp"
#define MODULE_DESCRIPTION	"Example Internet Security Association and Key Management protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	500	
#define MODULE_IDENT_ID2	500

#define MODULE_FILTER		"udp and port 500"

#define APSR_ISAKMP_FLAG_ENCRYPTION 	0x01
#define APSR_ISAKMP_FLAG_COMMIT		0x02
#define APSR_ISAKMP_FLAG_AUTH_ONLY	0x04

struct mod_isakmp_header {
	unsigned char initiator_cookie[8];
	unsigned char responder_cookie[8];
	unsigned int next_payload:8,
	 	     major_version:4,
		     minor_version:4,
		     exchange_type:8,
		     flags:8;
	unsigned int msg_id;
	unsigned int length;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_isakmp_header *isakmp_header;
  unsigned short i=0;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    isakmp_header = (struct mod_isakmp_header *)buf;
	    apsr_output("- ISAKMP Header -\n");
	    apsr_output("  Initiator Cookie: \"");
	    for(i = 0; i <= 7; i++) {
		    if(isprint(isakmp_header->initiator_cookie[i])) 
			    apsr_output("%c", 
				isakmp_header->initiator_cookie[i]);
		    else 
			    apsr_output(".");
	    }	 
            apsr_output("\"\n  Responder Cookie: \"");
            for(i = 0; i <= 7; i++) {
		    if(isprint(isakmp_header->responder_cookie[i]))
			    apsr_output("%c", 
			 	isakmp_header->responder_cookie[i]);
		    else
			    apsr_output(".");
            }
	    apsr_output("\"\n  Next Payload: %u, Major Version: %u, Minor Version: %u\n", 
			    isakmp_header->next_payload,
			    isakmp_header->major_version,
			    isakmp_header->minor_version);
	    apsr_output("  Exchange Type: %u, Flags: 0x%02x",
			    isakmp_header->exchange_type,
			    isakmp_header->flags);
	    if(verbose) {
		    apsr_output(" [");
		    if(isakmp_header->flags & APSR_ISAKMP_FLAG_ENCRYPTION)
			    apsr_output(" Encryption");
		    if(isakmp_header->flags & APSR_ISAKMP_FLAG_COMMIT) 
			    apsr_output(" Commit");
		    if(isakmp_header->flags & APSR_ISAKMP_FLAG_AUTH_ONLY)
			    apsr_output(" Authentication only");
		    apsr_output(" ]");
	    }
	    apsr_output("\n  Message ID: %u, Length: %u\n",
			    apsr_ntohl(isakmp_header->msg_id),
			    apsr_ntohl(isakmp_header->length));
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
