#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"ldp"
#define MODULE_DESCRIPTION	"Example Label Distribution Protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_PN2	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	646
#define MODULE_IDENT_ID2	646

#define MODULE_FILTER		"(tcp and udp) and port 646"

#define APSR_MOD_LDP_SPACE_NOTI		0x0001
#define APSR_MOD_LDP_SPACE_HELLO	0x0100	
#define APSR_MOD_LDP_SPACE_INIT		0x0200
#define APSR_MOD_LDP_SPACE_KEEPA	0x0201
#define APSR_MOD_LDP_SPACE_ADDRESS	0x0300
#define APSR_MOD_LDP_SPACE_ADDRESSW	0x0301
#define APSR_MOD_LDP_SPACE_LMAPP	0x0400
#define APSR_MOD_LDP_SPACE_LREQ		0x0401
#define APSR_MOD_LDP_SPACE_LWITH	0x0402
#define APSR_MOD_LDP_SPACE_LREL		0x0403
#define APSR_MOD_LDP_SPACE_LABOR	0x0404
#define APSR_MOD_LDP_SPACE_CSETUP	0x0500
#define APSR_MOD_LDP_SPACE_CREL		0x0501

struct mod_ldp_header {
	unsigned short version;
	unsigned short length;
	unsigned int lsr_id;
	unsigned short label_space;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_ldp_header *ldp_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    ldp_header = (struct mod_ldp_header *)buf;
	    apsr_output("- LDP Header -\n");
	    apsr_output("  Version: %u, Length: %u\n", 
			    apsr_ntohs(ldp_header->version),
			    apsr_ntohs(ldp_header->length));
	    apsr_output("  LSR ID: %u, Label space: %u",
			    apsr_ntohl(ldp_header->lsr_id),
			    apsr_ntohs(ldp_header->label_space));
	    if(verbose) {
		    switch(apsr_ntohs(ldp_header->label_space)) {
			    case APSR_MOD_LDP_SPACE_NOTI:
  			      apsr_output(" (Notification)");
			      break;
			    case APSR_MOD_LDP_SPACE_HELLO:
			      apsr_output(" (Hello)");
			      break;
			    case APSR_MOD_LDP_SPACE_INIT:
			      apsr_output(" (Initialization)");
			      break;
			    case APSR_MOD_LDP_SPACE_KEEPA:
			      apsr_output(" (KeepAlive)");
			      break;
			    case APSR_MOD_LDP_SPACE_ADDRESS:
			      apsr_output(" (Address)");
			      break;
			    case APSR_MOD_LDP_SPACE_ADDRESSW:
			      apsr_output(" (Address Withdraw)");
			      break;
			    case APSR_MOD_LDP_SPACE_LMAPP:
			      apsr_output(" (Label Mapping)");
			      break;
			    case APSR_MOD_LDP_SPACE_LREQ:
			      apsr_output(" (Label Request)");
			      break;
			    case APSR_MOD_LDP_SPACE_LWITH:
			      apsr_output(" (Label Withdraw)");
			      break;
			    case APSR_MOD_LDP_SPACE_LREL:
			      apsr_output(" (Label Release)");
			      break;
			    case APSR_MOD_LDP_SPACE_LABOR:
			      apsr_output(" (Label Abort Request)");
			      break;
			    case APSR_MOD_LDP_SPACE_CSETUP:
			      apsr_output(" (Call Setup)");
			      break;
			    case APSR_MOD_LDP_SPACE_CREL:
			      apsr_output(" (Call Release)");
			      break;
			    default:
			      apsr_output(" (Unknown)");
			      break;
		    }			      
	    }		    
  }			      

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
