#include <apsrlib/apsrlib.h>

/* TODO: Information Element header */

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"lfap"
#define MODULE_DESCRIPTION	"Example Light-weight Flow Admission Protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_ID1	3145	
#define MODULE_IDENT_ID2	3145

#define MODULE_FILTER		"tcp and port 3145"

static const char *mod_lfap_codes[]= {
	" (Unknown)",
	" (FAR)",
	" (FAA)",
	" (FAU)",
	" (FUN)",
	" (FUA)",
	" (FCR)",
	" (FCA)",
	" (AR)",
	" (ARA)",
};

static const char *mod_lfap_status[]= {
	" (Success)",
        " (Corrupted)",
        " (Version)",
};				

struct mod_lfap_header {
	unsigned char version;
	unsigned char code;
	unsigned char reserved;
	unsigned char status;
	unsigned short message_id;
	unsigned short message_length;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_lfap_header *lfap_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    lfap_header = (struct mod_lfap_header *)buf;
	    apsr_output("- LFAP Header -\n");
	    apsr_output("  Version: %u, Code %u", lfap_header->version,
			    lfap_header->code);
	    if(verbose)
		    apsr_output("%s", mod_lfap_codes[lfap_header->code]);
	    apsr_output("\n  Reserved: %u, Status: %u", lfap_header->reserved,
			    lfap_header->status);
	    if(verbose)
		    apsr_output("%s", mod_lfap_status[lfap_header->status]);
	    apsr_output("\n  Message ID: %u, Message Length: %u\n",
			    apsr_ntohs(lfap_header->message_id),
			    apsr_ntohs(lfap_header->message_length));
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
